#include "MERLIN_Plan.h"

#include <sstream>
using std::stringstream;

extern int merlin_client_debug_;

void MERLIN_Plan::planDEBUG(const string &message, const int &indentation_level)
{
    if (merlin_client_debug_)
    {
        for (int i=0; i < indentation_level; i++)
            cout<<"    ";
        cout<<message<<endl;
    }
}

string MERLIN_Plan::get_xml_declaration_type(const int &t)
{
    switch ( t )
    {
        case TiXmlNode::DOCUMENT:
            return string("TiXmlNode::DOCUMENT");
            break;
        case TiXmlNode::COMMENT:
            return string(" TiXmlNode::COMMENT");
            break;
        case TiXmlNode::UNKNOWN:
            return string ("TiXmlNode::UNKNOWN");
            break;
        case TiXmlNode::TEXT:
            return string("TiXmlNode::TEXT");
            break;
        case TiXmlNode::DECLARATION:
            return string("TiXmlNode::DECLARATION");
            break;
        case TiXmlNode::ELEMENT:
            return string("TiXmlNode::ELEMENT");
            break;
        default:
            return string("UNKNOWN");
            break;
    }
}

void MERLIN_Plan::load_ranges_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN loading RANGES", 2);
    int t;
    TiXmlNode* pChild = 0;
    TiXmlNode* pChild1 = 0;
    
    // We must loop for all RANGE in RANGES
    for ( pChild = pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        // We get sure this is a valid element
        if (t != TiXmlNode::ELEMENT)
        {
            string error = string("Invalid type of XML object, expected TiXmlNode::ELEMENT, got ") + get_xml_declaration_type(t);
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            
        }
        else // Yes, this is a valid element ...
        {
            if (string(pChild->Value()) != "RANGE") // ... but we must get sure its name is RANGE
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 2.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                
            }
            else // element inside RANGES is indeed named RANGE
            {
                planDEBUG("BEGIN RANGE", 3);
                // Here we check that an element RANGE has an attribute named 'index'
                // Please notice that so far this code is not enforcing that a RANGE
                // has only one attribute, all we are doing is checking the first attribute.
                TiXmlAttribute* pAttrib=((TiXmlElement*)pChild)->FirstAttribute();
                if (!pAttrib)
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "RANGE does not contain an attribute.");
                else
                {
                    if (string(pAttrib->Name()) != string("index"))
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "RANGE first attribute must be named 'index'.");  
                    else
                    {
                        string pair1 = string(pAttrib->Name()) + string(" = ") + string(pAttrib->Value());
                        planDEBUG(pair1, 3); 
                    }
                }
                
                // Start building the MERLIN_Range object.
                MERLIN_Range my_range;
                int ival;
                if (pAttrib->QueryIntValue(&ival) == TIXML_SUCCESS)
                    my_range.first = ival;
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to integer for RANGE index attribute.");  
                
                // There can be only one REGION inside a RANGE
                bool got_region = false;
                for ( pChild1 = pChild->FirstChild(); pChild1 != 0; pChild1 = pChild1->NextSibling()) 
                {
                    if (string(pChild1->Value()) != "REGION")
                    {
                        string error = string("Invalid XML element ") + string(pChild1->Value()) + string(" at level 2.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                        
                    }
                    else
                    {
                        if (got_region) // There can only be a REGION in a RANGE
                            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "There can not be more than one REGION per RANGE");
                        else
                        {
                            planDEBUG("BEGIN REGION", 4);
                            MERLIN_Region my_region;
                            TiXmlAttribute* pAttrib1=((TiXmlElement*)pChild1)->FirstAttribute();
                            while (pAttrib1)
                            {
                                string pair = string(pAttrib1->Name()) + string(" = ") + string(pAttrib1->Value());
                                planDEBUG(pair, 5);
                                if (string(pAttrib1->Name()) == string("base"))
                                {
                                    double dval;
                                    if (pAttrib1->QueryDoubleValue(&dval)==TIXML_SUCCESS)
                                    {
                                        my_range.wave_base = dval;
                                        my_region.first_wavelength = dval;
                                    }
                                    else
                                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to double for REGION 'base' attribute.");  
                                }
                                else if (string(pAttrib1->Name()) == string("dispersion"))
                                {
                                    double dval;
                                    if (pAttrib1->QueryDoubleValue(&dval)==TIXML_SUCCESS)
                                    {
                                        my_range.wave_step = dval;
                                        my_region.wave_step = dval;
                                    }
                                    else
                                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to double for REGION 'dispersion' attribute.");    
                                }
                                else if (string(pAttrib1->Name()) == string("number_steps"))
                                {
                                    int nsteps;
                                    if (pAttrib1->QueryIntValue(&nsteps) == TIXML_SUCCESS)
                                    {
                                        my_range.last = my_range.first + nsteps - 1;
                                        my_region.n_wavelengths = nsteps;
                                    }
                                    
                                    else
                                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to integer for REGION 'number_steps' attribute.");  
                                }
                                else
                                {
                                    string error = string("Unrecognized attribute ") + string(pAttrib1->Name()) + string (" for element REGION.");
                                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);                   
                                }
                                pAttrib1=pAttrib1->Next();
                            }
                            model.regions.push_back(my_region);
                            got_region = true;
                            planDEBUG("END REGION", 4);
                        } 
                    }
                } // End loop for all REGION inside a RANGE
                parsed_ranges.push_back(my_range);
                planDEBUG("END RANGE", 3);
            } // End for this RANGE
        } // Close else if we indeed got a TiXmlNode::ELEMENT
    } // End for all RANGE in RANGES
    
    // set the number of regions obtained in the model
    model.n_regions = model.regions.size();
    for( int region_index = 0; region_index < model.n_regions; region_index++ )
        model.n_total_wave += model.regions[region_index].n_wavelengths;
    planDEBUG("END loading RANGES", 2);
}

void MERLIN_Plan::load_lines_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN loading LINES", 2);
    
    int t;
    TiXmlNode* pChild = 0;
    for ( pChild = pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            string error = string("Invalid type of XML object, expected TiXmlNode::ELEMENT, got ") + get_xml_declaration_type(t);
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            
        }
        else
        {
            if(string(pChild->Value()) == string("ATOMIC_LINE"))
            {
                MERLIN_AtomicLine line;
             	TiXmlAttribute* pAttrib=((TiXmlElement*)pChild)->FirstAttribute();
                while (pAttrib)
                {
                    
                    string pair = string(pAttrib->Name()) + string(" = ") + string(pAttrib->Value());
                    planDEBUG(pair, 3);
                    if (string(pAttrib->Name()) == "lambda")
                    {
                        char **pt =0;
                        line.central_wav = strtod(pAttrib->Value(), pt);
                        
                    }
                    if (string(pAttrib->Name()) == "relative_strength")
                    {
                        char **pt =0;
                        line.relative_strength = strtod(pAttrib->Value(), pt);
                        
                    }
                    if (string(pAttrib->Name()) == "lower_ls_coupling")
                    {
                        MERLIN_LScoupling e_lower(pAttrib->Value()) ;
                        line.e_lower = e_lower ;
                    }
                    if (string(pAttrib->Name()) == "upper_ls_coupling")
                    {
                        MERLIN_LScoupling e_upper(pAttrib->Value()) ;
                        line.e_upper = e_upper ;
                    }
                    pAttrib=pAttrib->Next();
                }
                line.load_components();
                model.lines.push_back(line);
            }
            else
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 2.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    
    // set the number of lines in the model
    model.n_lines = model.lines.size();
    
    planDEBUG("END loading LINES", 2);
}

void MERLIN_Plan::load_stray_light_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN loading STRAY_LIGHT", 2);
    stringstream ss(pNode->FirstChild()->ToText()->Value());
    planDEBUG(pNode->FirstChild()->ToText()->Value(), 3);
    
    double data;
    std::vector<double> my_stray_light_vector;
    while (!ss.eof())
    {
        ss >> data;
        my_stray_light_vector.push_back(data);
    }
    int n = my_stray_light_vector.size();
    int register n2 =1;
    while (n2< n)
        n2 = n2 << 1;
    compute_smoooth_extension (my_stray_light_vector, n, n2);
    model.stray_light_profile.set_stray_light(my_stray_light_vector);
    planDEBUG("END loading STRAY_LIGHT", 2);
}

void MERLIN_Plan::load_parameters_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding PARAMETERS to plan", 1); 
    TiXmlNode* pChild = 0;
    int t;
    bool got_field = false;
    bool got_inclination = false;
    bool got_azimuth = false;
    bool got_line_strength = false;
    bool got_doppler_width = false;
    bool got_damping = false;
    bool got_source_function = false;
    bool got_source_function_gradient = false;
    bool got_macroturbulence = false;
    bool got_stray_light = false;
    bool got_stray_light_shift = false;
    bool got_doppler_shift_line1 = false;
    bool got_doppler_shift_line2 = false;
    if (model.n_lines != 2)
    {
        string error ("This code is designed for two atomic lines. Please get sure that in the input XML file, for the element <LINES> you have entered data for two atomic lines");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);        
    }
    
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            string error = string("Invalid type of XML object, expected TiXmlNode::ELEMENT, got ") + get_xml_declaration_type(t);
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
        }
        else
        {
            if (string(pChild->Value()) == string("FIELD_STRENGTH"))
            {
                if (!got_field)
                {
                    got_field = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.magnetic_field.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.magnetic_field.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
            }
            else if (string(pChild->Value()) == string("FIELD_INCLINATION"))
            {
                if (!got_inclination)
                {
                    got_inclination = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.inclination.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.inclination.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("FIELD_AZIMUTH"))
            {
                if (!got_azimuth)
                {
                    got_azimuth = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.azimuth.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.azimuth.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("LINE_STRENGTH"))
            {
                if (!got_line_strength)
                {
                    got_line_strength = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.line_strength.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.line_strength.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("DOPPLER_WIDTH"))
            {
                if (!got_doppler_width)
                {
                    got_doppler_width = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.doppler_width.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.doppler_width.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("DAMPING"))
            {
                if (!got_damping)
                {
                    got_damping = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.damping.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.damping.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("DOPPLER_WIDTH"))
            {
                if (!got_doppler_width)
                {
                    got_doppler_width = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.doppler_width.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.doppler_width.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("SOURCE_FUNCTION"))
            {
                if (!got_source_function)
                {
                    got_source_function = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.planck_function.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.planck_function.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("SOURCE_FUNCTION_GRADIENT"))
            {
                if (!got_source_function_gradient)
                {
                    got_source_function_gradient = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.planck_function_gradient.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.planck_function_gradient.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("MACROTURBULENCE"))
            {
                if (!got_macroturbulence)
                {
                    got_macroturbulence = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.vmac.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.vmac.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("STRAY_LIGHT"))
            {
                if (!got_stray_light)
                {
                    got_stray_light = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.stray_light.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.stray_light.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("STRAY_LIGHT_SHIFT"))
            {
                if (!got_stray_light_shift)
                {
                    got_stray_light_shift = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.stray_light_shift.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.stray_light_shift.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("DOPPLER_SHIFT_LINE1"))
            {
                if (!got_doppler_shift_line1)
                {
                    got_doppler_shift_line1 = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.lines[0].wavelength_shift.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.lines[0].wavelength_shift.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else if (string(pChild->Value()) == string("DOPPLER_SHIFT_LINE2"))
            {
                if (!got_doppler_shift_line2)
                {
                    got_doppler_shift_line2 = true;
                    string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                    planDEBUG(pair, 2);
                    if (pChild->FirstChild()->ToText()->Value() == string("true"))
                        model.lines[1].wavelength_shift.free = true;
                    else if (pChild->FirstChild()->ToText()->Value() == string("false"))
                        model.lines[1].wavelength_shift.free = false;
                    else
                    {
                        string error = string("Value must be either 'true' or false'.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the PARAMETERS.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    
                }
                
            }
            else
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 1.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    if (!got_field)
    {
        string error = string("Missing XML element FIELD_STRENGTH.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_inclination)
    {
        string error = string("Missing XML element FIELD_INCLINATION.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_azimuth)
    {
        string error = string("Missing XML element FIELD_AZIMUTH.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_line_strength)
    {
        string error = string("Missing XML element LINE_STRENGTH.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_doppler_width)
    {
        string error = string("Missing XML element DOPPLER_WIDTH.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_damping)
    {
        string error = string("Missing XML element DAMPING.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_source_function)
    {
        string error = string("Missing XML element SOURCE_FUNCTION.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_source_function_gradient)
    {
        string error = string("Missing XML element SOURCE_FUNCTION_GRADIENT.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_macroturbulence)
    {
        string error = string("Missing XML element MACROTURBULENCE.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    if (!got_stray_light)
    {
        string error = string("Missing XML element STRAY_LIGHT.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    }  
    if (!got_stray_light_shift)
    {
        string error = string("Missing XML element STRAY_LIGHT_SHIFT.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    }  
    if (!got_doppler_shift_line1)
    {
        string error = string("Missing XML element DOPPLER_SHIFT_LINE1.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    }  
    if (!got_doppler_shift_line2)
    {
        string error = string("Missing XML element DOPPLER_SHIFT_LINE2.");
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
    } 
    planDEBUG("END adding PARAMETERS to plan", 1); 
}

void MERLIN_Plan::load_model_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding MODEL to plan", 1); 
    TiXmlNode* pChild = 0;
    int t;
    
    // With this boolean flags we get sure that there is only one 
    // of each of the elements RANGES,LINES and STRAY_LIGHT. 
    bool loaded_ranges = false;
    bool loaded_lines = false;
    bool loaded_stray_light = false;
    
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            string error = string("Invalid type of XML object, expected TiXmlNode::ELEMENT, got ") + get_xml_declaration_type(t);
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
        }
        else
        {
            if(string(pChild->Value()) == string("RANGES"))
            {
                if (!loaded_ranges)
                {
                    load_ranges_from_xml_object(pChild);
                    loaded_ranges = true;
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the MODEL.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                }
            }
            else if (string(pChild->Value()) == string("LINES"))
            {
                if (!loaded_lines)
                {
                    load_lines_from_xml_object(pChild);
                    loaded_lines = true;
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the MODEL.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                }
            }
            else if  (string(pChild->Value()) == string("STRAY_LIGHT"))
            {
                if (!loaded_stray_light)
                {
                    load_stray_light_from_xml_object(pChild);
                    loaded_stray_light = true;
                }
                else
                {
                    string error = string("Element of type ") + string (pChild->Value()) 
                    + string (" is already loaded and can't be repeated inside the MODEL.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                }
            }
            else
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 1.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    if (!loaded_ranges)
    {
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element RANGES.");
    }
    else
    {
        model.stray_light_profile.set_ranges(parsed_ranges);
    }
    if (!loaded_lines)
    {
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element LINES.");
    }
    // Notice that we do not get sure to have a stray light vector as
    // it is possible to do an inversion without one.
    
    planDEBUG("END adding MODEL to plan", 1); 
    
}

void MERLIN_Plan::load_genetic_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding GENETIC to plan", 1); 
    TiXmlNode* pChild = 0;
    int t;
    
    bool got_individuals = false;
    bool got_generations = false;
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            cerr<<__FILE__<<":"<<__LINE__<<": Invalid type of XML object, expected TiXmlNode::ELEMENT, got ";
            get_xml_declaration_type(t);
            exit(1);
        }
        else
        {
            string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
            planDEBUG(pair, 2);
            
            if (string(pChild->Value()) == string("individuals"))
            {
                if(!got_individuals)
                {
                    genetic.individuals = atoi(pChild->FirstChild()->ToText()->Value());
                    got_individuals = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "'individuals' is repeated.");
            }
            else if (string(pChild->Value()) == string("generations"))
            {
                if(!got_generations)
                {
                    genetic.generations = atoi(pChild->FirstChild()->ToText()->Value());
                    got_generations= true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "'generations' is repeated.");
                
            }
            else
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 1.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    planDEBUG("END adding GENETIC to plan", 1);
}

void MERLIN_Plan::load_weights_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding PARAMETERS to plan", 1); 
    TiXmlNode* pChild = 0;
    int t;
    
    bool got_stokes_I = false;
    bool got_stokes_Q = false;
    bool got_stokes_U = false;
    bool got_stokes_V = false;
    
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            cerr<<__FILE__<<":"<<__LINE__<<": Invalid type of XML object, expected TiXmlNode::ELEMENT, got ";
            get_xml_declaration_type(t);
            exit(1);
        }
        else
        {
            string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
            planDEBUG(pair, 2);
            
            if (string(pChild->Value()) == string("STOKES_I"))
            {
                if(!got_stokes_I)
                {
                    char **pt=0;
                    CSAC::STOKES_I_WEIGHT = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_stokes_I = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "WEIGHTS 'STOKES_I' is repeated.");
            }
            else if (string(pChild->Value()) == string("STOKES_Q"))
            {
                if(!got_stokes_Q)
                {
                    char **pt=0;
                    CSAC::STOKES_Q_WEIGHT = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_stokes_Q = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "WEIGHTS 'STOKES_Q' is repeated.");
                
            }
            else if (string(pChild->Value()) == string("STOKES_U"))
            {
                if(!got_stokes_U)
                {
                    char **pt=0;
                    CSAC::STOKES_U_WEIGHT = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_stokes_U = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "WEIGHTS 'STOKES_U' is repeated.");
                
            }
            else if (string(pChild->Value()) == string("STOKES_V"))
            {
                if(!got_stokes_V)
                {
                    char **pt=0;
                    CSAC::STOKES_V_WEIGHT = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_stokes_V = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "WEIGHTS 'STOKES_V' is repeated.");
                
            }
            else
            {
                string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 1.");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    planDEBUG("END adding WEIGHTS to plan", 1);
}

void MERLIN_Plan::load_files_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding FILES to plan", 1);
    TiXmlNode* pChild = 0;
    int t;
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
    {
        
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            cerr<<__FILE__<<":b"<<__LINE__<<": Invalid type of XML object, expected TiXmlNode::ELEMENT, got ";
            get_xml_declaration_type(t);
            exit(1);
        }
        else
        {
            if (string(pChild->Value()) == string("INPUT"))
            {
                string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                planDEBUG(pair, 2);    
                input_file = pChild->FirstChild()->ToText()->Value();
                
            }
            else if (string(pChild->Value()) == string("INIT"))
            {
                string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                planDEBUG(pair, 2);
                guess_file = pChild->FirstChild()->ToText()->Value();
            }
            else if (string(pChild->Value()) == string("OUTPUT"))
            {
                string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
                planDEBUG(pair, 2);
                output_file = pChild->FirstChild()->ToText()->Value();
            }
            else if (string(pChild->Value()) == string("SLIT_POSITION"))
            {
                TiXmlAttribute* pAttrib1=((TiXmlElement*)pChild)->FirstAttribute();
                while (pAttrib1)
                {
                    string pair = string(pAttrib1->Name()) + string(" = ") + string(pAttrib1->Value());
                    planDEBUG(pair, 2);
                    if (string(pAttrib1->Name()) == string("Y_min"))
                    {
                        int val;
                        if (pAttrib1->QueryIntValue(&val)==TIXML_SUCCESS)
                        {
                            y_min = val;
                        }
                        else
                            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to integer for FILES|SLIT_POSITION 'Y_min' attribute.");  
                    }
                    else if (string(pAttrib1->Name()) == string("Y_max"))
                    {
                        int val;
                        if (pAttrib1->QueryIntValue(&val)==TIXML_SUCCESS)
                        {
                            y_max = val;
                        }
                        else
                            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Failed to cast string to double for FILES|SLIT_POSITION 'Y_max' attribute.");    
                    }
                    else
                    {
                        string error = string("Unrecognized attribute ") + string(pAttrib1->Name()) + string (" for FILES|SLIT_POSITION element .");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);                   
                    }
                    pAttrib1=pAttrib1->Next();
                    
                }
            }
            else
            {
                string error = string("INVALID element ") + string(pChild->Value()) + string(" in FILES");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    planDEBUG("END adding FILES to plan", 1);
}

void MERLIN_Plan::load_limits_from_xml_object(TiXmlNode* pNode)
{
    planDEBUG("BEGIN adding LIMITS to plan", 1);
    
    TiXmlNode* pChild = 0;
    // bool got_SVD_threshold = false; Removed SVD NR code
    bool got_worst_tolerated_condition_number = false;
    bool got_max_damping_parameter = false;
    bool got_min_damping_parameter = false;
    bool got_max_number_failures = false;
    bool got_max_number_iterations = false;
    bool got_polarization_threshold = false;
    bool got_stray_light_model = false;
    bool got_solver = false;
    int t;
    
    for ( pChild =  pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        t = pChild->Type();
        if (t != TiXmlNode::ELEMENT)
        {
            cerr<<__FILE__<<":"<<__LINE__<<": Invalid type of XML object, expected TiXmlNode::ELEMENT, got ";
            get_xml_declaration_type(t);
            exit(1);
        }
        else
        {
            string pair = string(pChild->Value()) + string(" = ") + string(pChild->FirstChild()->ToText()->Value());
            planDEBUG(pair, 2);
            
            if (string(pChild->Value()) == string("SVD_threshold"))
            {
				if (merlin_client_debug_)
					cout<<"Deprecated SVD_threshold keyword found in XML file, ignoring."<<endl;
				/* Ignore SVD_threshold keyword
                if(!got_SVD_threshold)
                {
                    char **pt=0;
                    CSAC::SVD_threshold = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_SVD_threshold = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'SVD_threshold' is repeated.");
				*/
            } else if ((string(pChild->Value()) == string("worst_tolerated_condition_number")))
            {
                if(!got_worst_tolerated_condition_number)
                {
                    char **pt=0;
                    CSAC::worst_tolerated_condition_number = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_worst_tolerated_condition_number = true;  
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'worst_tolerated_condition_number' is repeated.");
            }
            else if ((string(pChild->Value()) == string("max_damping_parameter")))
            {
                if(!got_max_damping_parameter)
                {
                    char **pt=0; 
                    CSAC::max_damping_parameter = strtod(pChild->FirstChild()->ToText()->Value(), pt);;
                    got_max_damping_parameter = true;
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'max_damping_parameter' is repeated.");
                
            }
            else if ((string(pChild->Value()) == string("min_damping_parameter")))
            {
                if(!got_min_damping_parameter)
                {
                    char **pt=0;
                    CSAC::min_damping_parameter = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_min_damping_parameter = true;
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'min_damping_parameter' is repeated.");
                
            }
            else if ((string(pChild->Value()) == string("max_number_failures")))
            {
                if(!got_max_number_failures)
                {
                    CSAC::max_number_failures = atoi(pChild->FirstChild()->ToText()->Value());
                    got_max_number_failures = true;
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'max_number_failures' is repeated.");
                
            }
            else if ((string(pChild->Value()) == string("max_number_iterations")))
            {
                if(!got_max_number_iterations)
                {
                    CSAC::max_number_iterations= atoi(pChild->FirstChild()->ToText()->Value());
                    got_max_number_iterations = true;
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'max_number_iterations' is repeated.");
            }
            else if ((string(pChild->Value()) == string("polarization_threshold")))
            {
                if(!got_polarization_threshold)
                {
                    char **pt=0;
                    polarization_threshold = strtod(pChild->FirstChild()->ToText()->Value(), pt);
                    got_polarization_threshold = true;
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'polarization_threshold' is repeated.");
            }
            else if ((string(pChild->Value()) == string("stray_light_model")))
            {
                if(!got_stray_light_model)
                {
                    string type_of_model = pChild->FirstChild()->ToText()->Value();
                    if (type_of_model == "FILL_FACTOR")
                    {
                        CSAC::stray_light_model = CSAC::fill;
                        got_stray_light_model = true;
                    }
                    else if (type_of_model == "FRACTION")
                    {
                        CSAC::stray_light_model =  fraction;
                        got_stray_light_model = true;
                    }
                    
                    else
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "'stray_light_model' can only be equal to either 'FILL_FACTOR' or 'FRACTION'");
                    
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'stray_light_model' is repeated.");
            }
            else if ((string(pChild->Value()) == string("solver")))
            {
                if(!got_solver)
                {
                    string type_of_solver = pChild->FirstChild()->ToText()->Value();
		    /*
		    // SVD NR code removed
                    if (type_of_solver == "SVD")
                    {
                        solver = "SVD";
                        got_solver = true;
                    }
		    */
                    if (type_of_solver == "Stepwise_Regression")
                    {
                        solver =  "SWR";
                        got_solver = true;
                    } else
                        // throw MERLIN_Plan_Exception(__FILE__, __LINE__, "'solver' can only be equal to either 'SVD' or 'Stepwise_Regression'");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, "'solver' can only be equal to 'Stepwise_Regression'");
                }
                else
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, "LIMITS 'solver' is repeated.");
            }
            else
            {
                string error = string("INVALID element ") + string(pChild->Value()) + string(" in LIMITS");
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
        }
    }
    
    planDEBUG("END adding LIMITS to plan", 1);
}

void MERLIN_Plan::build_inversion_plan_from_xml_file( const char* pFilename)
{
    planDEBUG("BEGIN building MERLIN_Plan", 0);
    
    TiXmlDocument doc(pFilename);
    
    if ( doc.LoadFile() )
    {
        
        // We start by checking this is indeed a XML document.
        int t = doc.Type();
        switch ( t )
        {
            case TiXmlNode::DOCUMENT:
                planDEBUG(string("XML document starts with proper tag ") + get_xml_declaration_type(t), 1);
                break;
            default:
            {
                string error = string("Invalid type of XML object, expected TiXmlNode::DOCUMENT, got ") + get_xml_declaration_type(t);
                throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
            }
                break;
        }
        
        TiXmlNode* pParent = &doc;
        TiXmlNode* pChild = 0;
        
        // This should be the XML declaration
        pChild = pParent->FirstChild();
        t = pChild->Type();
        if (t != TiXmlNode::DECLARATION)
        {
            string error = string("Invalid type of XML object, expected TiXmlNode::DECLARATION, got ") + get_xml_declaration_type(t);
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
        }
        string decl = string("xml version = ") + string(((TiXmlDeclaration*)pChild)->Version());
        decl += string(" encoding = ") + string(((TiXmlDeclaration*)pChild)->Encoding());
        planDEBUG(decl, 1);    
        
        // These boolean variables ensure that only one
        // of each of the elements MODEL, FILES and LIMITS
        // exist in the XML document
        bool loaded_model = false;
        bool loaded_files = false;
        bool loaded_limits = false;
        bool loaded_parameters = false;
        bool loaded_weights = false;
        bool loaded_genetic = false;
        
        pChild =  pParent->FirstChild()->NextSibling();
        if (string(pChild->Value()) != string("INVERSION"))
        {
            string error = string("Root Element ") + string (pChild->Value()) 
            + string (" does not match expected value \"INVERSION\"");
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
        }
        
        decl = string("Found root element: ") + string(pChild->Value());
        planDEBUG(decl, 1);
        
        for ( pChild =  pChild->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
        {
            t = pChild->Type();
            if (t != TiXmlNode::ELEMENT)
            {
                cerr<<__FILE__<<":"<<__LINE__<<": Invalid type of XML object, expected TiXmlNode::ELEMENT, got ";
                get_xml_declaration_type(t);
                exit(1);
            }
            else
            {
                if (string(pChild->Value()) == string("MODEL"))
                {
                    if (!loaded_model)
                    {
                        load_model_from_xml_object(pChild);
                        loaded_model = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else if (string(pChild->Value()) == string("PARAMETERS"))
                {
                    if (!loaded_parameters)
                    {
                        load_parameters_from_xml_object(pChild);
                        loaded_parameters = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                }
                else if  (string(pChild->Value()) == string("FILES"))
                {
                    if (!loaded_files)
                    {
                        load_files_from_xml_object(pChild);
                        loaded_files = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                    
                }
                else if  (string(pChild->Value()) == string("LIMITS"))
                {
                    if (!loaded_limits)
                    {
                        load_limits_from_xml_object(pChild);
                        loaded_limits = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                    
                }
                else if  (string(pChild->Value()) == string("WEIGHTS"))
                {
                    if (!loaded_weights)
                    {
                        load_weights_from_xml_object(pChild);
                        loaded_weights = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                    
                }         
                else if  (string(pChild->Value()) == string("GENETIC"))
                {
                    if (!loaded_genetic)
                    {
                        load_genetic_from_xml_object(pChild);
                        loaded_genetic = true;
                    }
                    else
                    {
                        string error = string("Element of type ") + string (pChild->Value()) 
                        + string (" is already loaded and can't be repeated inside the XML document.");
                        throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                    }
                    
                }         
                else
                {
                    string error = string("Invalid XML element ") + string(pChild->Value()) + string(" at level 0.");
                    throw MERLIN_Plan_Exception(__FILE__, __LINE__, error);
                }
            }
        }
        if (!loaded_parameters)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element PARAMETERS.");
        }
        if (!loaded_model)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element MODEL.");
        }
        if (!loaded_files)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element FILES.");
        }
        if (!loaded_limits)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element LIMITS.");
        }
        if (!loaded_weights)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element WEIGHTS.");
        }
        if (!loaded_genetic)
        {
            throw MERLIN_Plan_Exception(__FILE__, __LINE__, "Missing XML element GENETIC.");
        }
    }
    else
    {
        string err = string ("Failed to load XML file ") + string(pFilename) + "\n";
        err += "TinyXML library returns: " + string(doc.ErrorDesc()) + "\n";
        char tmp[32];
        snprintf(tmp, sizeof(tmp), "%i", doc.ErrorRow());
        err += "Error row: " + string(tmp);
        snprintf(tmp, sizeof(tmp), "%i", doc.ErrorCol());
        err += ", Error column: " + string(tmp) + "\n";     
        throw MERLIN_Plan_Exception(__FILE__, __LINE__, err);
    }
    planDEBUG("END building MERLIN_Plan", 0);
}


void MERLIN_Plan::compute_smoooth_extension (vector <double> &data, int original, int desired)
{
    double arg;
    int extension_size = desired - original;
    int hh=0;
    for (int i=original; i <desired ; i++)
    {
        //arg = .5 * (1 -  cos((1:extension)*pi/extension))
        arg = .5 * (1 - cos((hh+1) * CSAC::Pi / extension_size));
        hh++;
        //signal(columns+1:my_pow) = arg * signal(1) + (1-arg) * signal(columns);
        data.push_back( ( arg * data[0] + (1-arg) * data[original -1]) );
    }
    
}

