/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Error.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include <time.h>
#include <string.h>

#include <iostream>
using std::cerr;

#include "MERLIN_Error.h"
using CSAC::MERLIN_Error;

#include "MERLIN_Exception.h"
using CSAC::MERLIN_Exception;

// Comment this definition if you do not want to use exceptions.
#define MERLIN_USE_EXCEPTIONS

void MERLIN_Error::errorT(int error, char *format, ...)
{
    MERLIN_Exception exception;
    if (format)
    {
        const time_t sctime=time(NULL);
        const struct tm *sttime=localtime(&sctime); 
        exception = exception + "MERLIN: Error time; ";
        exception = exception + asctime(sttime);
        int count=0;
        int total_number_strings = strlen(format);
        va_list arg_ptr;
        va_start(arg_ptr,format);
        while(count<total_number_strings)
        {
            // character and float are not supported because of data conversion before the call for
            // arguments for which there is not parameter present in the function prototype
            // See "The C++ Programming Language" Bjarne Stroustrup Second Edition.
            // Pag.495, section r.5.2.2
            switch(format[count])
            {
                case 'c':
                    /* need a cast here since va_arg only
                       takes fully promoted types */
                    exception = exception + (char) va_arg(arg_ptr, int);
                    break;
                case 's':
                    exception = exception + va_arg(arg_ptr, char*);
                    break;
                case 'i':
                    exception = exception + va_arg(arg_ptr, int);
                    break;
                case 'd':
                    exception = exception + va_arg(arg_ptr, double);
                    break;
                default:
                    // If we get an unrecognize character we terminate the loop.
                    count=total_number_strings;
                    break;
            }
            count++;
        }
        va_end(arg_ptr);
    }
    exception.set_error_code(error);
#ifdef MERLIN_USE_EXCEPTIONS
    throw exception;
#else
    cerr << exception;
    exit( exception.get_error_code() );
#endif // MERLIN_USE_EXCEPTIONS
}

