#ifndef MERLIN_FFTW_ST_h_
#define MERLIN_FFTW_ST_h_ 1

#include <string>
#include <iostream>
#include <vector>

#include <string.h>
#include <math.h>
#include <fftw3.h>

using namespace std;

/**
 */

class MERLIN_FFTW_PLAN
{
    ///
    fftw_plan __plan;
    ///
    double *__internal_buffer;
    ///
    int __plan_size;
public:
    ///
    MERLIN_FFTW_PLAN(int size)
    {
#ifdef DEBUG_MERLIN_FFTW
        cout<<"Constructing plan for size "<<size<<endl;
#endif
        __plan_size=size;
        __internal_buffer = new double [size];
        __plan = fftw_plan_r2r_1d(size, __internal_buffer, __internal_buffer, FFTW_R2HC, FFTW_MEASURE);
    }
    /// Copy constructor
    MERLIN_FFTW_PLAN(const MERLIN_FFTW_PLAN &rhs)throw()
    {
#ifdef DEBUG_MERLIN_FFTW
        cout<<"Constructing with copy constructor plan for size "<<rhs.__plan_size<<endl;
#endif
        this->__plan_size = rhs.__plan_size;
        this->__internal_buffer = new double [this->__plan_size];
        memcpy((void*) this->__internal_buffer, (void*) rhs.__internal_buffer, (size_t) (this->__plan_size * sizeof(double)));
        __plan = fftw_plan_r2r_1d(this->__plan_size, this->__internal_buffer, this->__internal_buffer, FFTW_R2HC, FFTW_MEASURE);
    };
    ///
    ~MERLIN_FFTW_PLAN()
    {
#ifdef DEBUG_MERLIN_FFTW
        cout<<"Destructing plan for size "<<__plan_size<<endl;
#endif
        fftw_destroy_plan(__plan);
        delete [] __internal_buffer;
    }
    ///
    void execute_plan(double *input, double *output)
    {
        ///
        memcpy((void*)__internal_buffer,(const void*)input, (size_t)(__plan_size * sizeof(double)));
        // Execute FFT 
        fftw_execute(__plan);
        // copy the data out
        memcpy((void*)output,(const void*)__internal_buffer, (size_t)(__plan_size * sizeof(double)));
#ifdef DEBUG_MERLIN_FFTW
        cout<<"Done using plan of size "<<__plan_size<<endl;
#endif
    }
    ///
    int size()
    {
        return __plan_size;
    }

};





/**
 */

class MERLIN_FFTW_ST
{
    ///
    static vector<MERLIN_FFTW_PLAN*> __the_plans;
    ///
    static MERLIN_FFTW_ST *__singletonRef;
    ///
    MERLIN_FFTW_ST() throw(){};
    ///
    MERLIN_FFTW_ST(const MERLIN_FFTW_ST&)throw(){};
    ///
    void operator delete(void*)throw(){};

public:
  
    ///
    static const MERLIN_FFTW_ST* instanceOf() throw()
    {
        if (!__singletonRef) 
        {
            __singletonRef = new MERLIN_FFTW_ST();
#ifdef DEBUG_MERLIN_FFTW
            cout<<"Creating the instance of MERLIN_FFTW_ST"<<endl;
#endif
        }
#ifdef DEBUG_MERLIN_FFTW
        else
            cout<<"Instance of MERLIN_FFTW_ST already exist!"<<endl;
#endif
        return __singletonRef;
    }

    ///
    const void execute (double *input, int input_size, double *output) const throw();
};

#endif //  MERLIN_FFTW_ST_h_
