/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Globals.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include "MERLIN.h"

// Global variables within the namespace MERLIN.
namespace CSAC
{
    // Read documentation for this variable in the header file MERLIN.h
    bool merlin_debug = false;
    // Read documentation for this variable in the header file MERLIN.h
    // double SVD_threshold = 1E-14;
    // Read documentation for this variable in the header file MERLIN.h
    double worst_tolerated_condition_number = 1E16;   
    // Read documentation for this variable in the header file MERLIN.h
    int max_number_failures = 6;
    // Read documentation for this variable in the header file MERLIN.h
    double max_damping_parameter = 1E8;
    // Read documentation for this variable in the header file MERLIN.h
    double min_damping_parameter = 1E-12;
    // Read documentation for this variable in the header file MERLIN.h
    int max_number_iterations = 70;
    // Read documentation for this variable in the header file MERLIN.h 
    const merlin_real perturbation = 1E-9;
    // Read documentation for this variable in the header file MERLIN.h 
    stray_light_model_type stray_light_model = fill;

    // Physical constants
    // Speed of light (m/s) 
    const merlin_real CC = 299792458 ; 
    // Pi 
    const merlin_real Pi = atan((merlin_real)1.0)*4.0;  
    // Used in the computation of the Faraday function. 
    // Please refer to void mvoigt_d(...etc...) in mvoigt.cc
    const merlin_real Piis = 1/sqrt(Pi); 
    
    double STOKES_I_WEIGHT = 1E-2; 
    
    double STOKES_Q_WEIGHT = 1.0;
    
    double STOKES_U_WEIGHT = 1.0;
    
    double STOKES_V_WEIGHT = 1E-1;
};


