/*
 *  MERLIN_ModelCorrection.cc
 *  MERLIN
 *
 *  Created by Jose Garcia on 5/18/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "MERLIN_ModelCorrection.h"
#include <iostream>

using CSAC::MERLIN_Model;
using CSAC::MERLIN_ParametersLimits;
using CSAC::MERLIN_ModelCorrection;
using CSAC::MERLIN_Parameter;
using namespace std;
//using CSAC::MERLIN_Inversion;
//using CSAC::MERLIN_MilneEddington;
//using CSAC::MERLIN_StokesProfile;
//using CSAC::MERLIN_SVDsolve;
//using CSAC::MERLIN_ParametersLimits;
//using CSAC::MERLIN_ModelCorrection;

void MERLIN_ModelCorrection::correct_trial_model(MERLIN_Model &trial, const MERLIN_ParametersLimits &limits) 
{
    
    // // Correct the inclination of the magnetic field vector
    if (trial.inclination.free)
    {
        if (trial.inclination.value < 0)
        {
            //cout<<"Clipping field inclination (too low)"<<endl;
            trial.inclination.value = trial.inclination.value * -1;
        }
        
        if (trial.inclination.value > CSAC::Pi)
        {
            double pi_in_inclination = trial.inclination.value/ CSAC::Pi;
            if (CSAC::merlin_debug) 
            {
                cout.precision(25);
                cout<<"Clipping field inclination (too high)"<<endl;
                cout<<"Got "<<trial.inclination.value<<endl;
                cout<<"Pi in inclination is "<<pi_in_inclination<<endl;
            }
            double integral_part;
            double fractional_part = modf(pi_in_inclination, &integral_part);
            if (CSAC::merlin_debug)
                cout<<"Fractional Part is "<<fractional_part<<" and integral part is "<<integral_part<<endl;
            trial.inclination.value = trial.inclination.value - (integral_part * CSAC::Pi);
            if (CSAC::merlin_debug)
                cout<<"Set "<<trial.inclination.value<<endl;
        }
    }
    // Correct the azimuth of the magnetic field vector
    if (trial.azimuth.free) 
    {
        if (trial.azimuth.value < 0)
        { 
            //cout<<"Clipping field azimuth (too low)"<<endl;
            double pi_in_azimuth = trial.azimuth.value/ CSAC::Pi;
            double integral_part;
            double fractional_part = modf(pi_in_azimuth, &integral_part);
            trial.azimuth.value = trial.azimuth.value - (integral_part * CSAC::Pi);
            // Still now we have a value between -180 degress (Pi) and 0
            // make positive the azimuth
            trial.azimuth.value += CSAC::Pi;
        }
        if (trial.azimuth.value > CSAC::Pi)
        {
            //cout<<"Clipping field azimuth (too high)"<<endl;
            double pi_in_azimuth = trial.azimuth.value/ CSAC::Pi;
            double integral_part;
            double fractional_part = modf(pi_in_azimuth, &integral_part);
            trial.azimuth.value = trial.azimuth.value - (integral_part * CSAC::Pi);
        }
    } 
    // Correct the strength of the magnetic field vector
    if (trial.magnetic_field.free)
    {
        if (trial.magnetic_field.value < 0)
        {
            //cout<<"Clipping field intensity (too low)"<<endl;
            trial.magnetic_field.value = trial.magnetic_field.value * -1;
            trial.inclination.value = CSAC::Pi - trial.inclination.value;
        }
        if (trial.magnetic_field.value > limits._max_magnetic_field)
        {
            //cout<<"Clipping field intensity (too high)"<<endl;
            trial.magnetic_field.value = limits._max_magnetic_field;
        }
    }
    
    if (trial.line_strength.free)
    {
        if (trial.line_strength.value < limits._min_line_strength)
        {
            //cout<<"Clipping line strength (too low)"<<endl;
            trial.line_strength.value = limits._min_line_strength;
        }
        if (trial.line_strength.value > limits._max_line_strength)
        {
            //cout<<"Clipping line strength (too high)"<<endl;
            trial.line_strength.value = limits._max_line_strength;
        }
    }
    
    if (trial.doppler_width.free)
    {
        if (trial.doppler_width.value < limits._min_doppler_width)
        {
            //cout<<"Clipping doppler width (too low)"<<endl;
            trial.doppler_width.value = limits._min_doppler_width;
        }
        if (trial.doppler_width.value > limits._max_doppler_width)
        {
            //cout<<"Clipping doppler width (too high)"<<endl;
            trial.doppler_width.value = limits._max_doppler_width; 
        }
    }
    
    if (trial.damping.free)
    {
        if (trial.damping.value < limits._min_damping)
        {
            //cout<<"Clipping damping (too low)"<<endl;
            trial.damping.value = limits._min_damping;
        }
        if (trial.damping.value > limits._max_damping)
        {
            //cout<<"Clipping damping (too high)"<<endl;
            trial.damping.value = limits._max_damping;
        }
    }
    
    for (int kk=0; kk < trial.lines.size(); kk++)
    {
        if (trial.lines[kk].wavelength_shift.free)
        {
            if (trial.lines[kk].wavelength_shift.value < -10)
            {
                //cout<<"Clipping wave lenght shift (too low) for atomic line "<<trial.lines[kk].central_wav<<endl;
                trial.lines[kk].wavelength_shift.value = -10;
            }
            if (trial.lines[kk].wavelength_shift.value > 10)
            {
                //cout<<"Clipping wave lenght shift (too high) for atomic line "<<trial.lines[kk].central_wav<<endl;
                trial.lines[kk].wavelength_shift.value = 10;
            }
        }
    }
    
    // Check to get sure the limits for B0 and B1 are set
    if(limits.set())
    {
        if (trial.planck_function.free)
        {
            if (trial.planck_function.value < limits._min_planck_function)
            {
                //cout<<"Clipping planck function (too low)"<<endl;
                trial.planck_function.value = limits._min_planck_function;
            }
            if (trial.planck_function.value > limits._max_planck_function)
            {
                //cout<<"Clipping planck function (too high)"<<endl;
                trial.planck_function.value = limits._max_planck_function;
            }
        }
        
        if (trial.planck_function_gradient.free)
        {
            if (trial.planck_function_gradient.value < limits._min_planck_function_gradient)
            {
                //cout<<"Clipping planck function gradient (too low)"<<endl;
                trial.planck_function_gradient.value = limits._min_planck_function_gradient;
            }
            if (trial.planck_function_gradient.value > limits._max_planck_function_gradient)
            {
                //cout<<"Clipping planck function gradient (too high)"<<endl;
                trial.planck_function_gradient.value = limits._max_planck_function_gradient;
            }
        }
    }
    else
        throw string("STOP! Inversion Limits have not been set up correctly.");
    
    if ( trial.vmac.free)
    {
        if (trial.vmac.value < limits._min_vmac)
        {
            //cout<<"Clipping macroturbulence (too low)"<<endl;
            trial.vmac.value = limits._min_vmac;
        }
        if (trial.vmac.value > limits._max_vmac)
        {
            //cout<<"Clipping macroturbulence (too high)"<<endl;
            trial.vmac.value = limits._max_vmac;
        }
    }
    
    if (trial.stray_light.free)
    {
        //cout<<"Before "<<trial.stray_light.value<<endl;
        if (trial.stray_light.value < 0.0)
        {
            //cout<<"Clipping stray light (too low)"<<endl;
            trial.stray_light.value = 0.0;
        }
        if (trial.stray_light.value > 1.0)
        {
            //cout<<"Clipping stray light (too high)"<<endl;
            trial.stray_light.value = 1.0;
        }
        //cout<<"After "<<trial.stray_light.value<<endl;
    }
    
    if (trial.stray_light_shift.free)
    {
        if (trial.stray_light_shift.value < -10.0)
        {
            //cout<<"Clipping stray light shift (too low)"<<endl;
            trial.stray_light_shift.value = -10.0;
        }
        if (trial.stray_light_shift.value > 10.0)
        {
            //cout<<"Clipping stray light shift (too high)"<<endl;
            trial.stray_light_shift.value = 10.0;
        }
    } 
}

