/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Object.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


 

#ifndef MERLIN_Object_h
#define MERLIN_Object_h 1

#include <iostream>
using std::ostream;

namespace CSAC
{
    /// \brief Base class for MERLIN objects to house generic methods.
    ///
    /// This class is used to allow all objects to dump their contents to a stream, such as standard output
    /// This is heavily user in the devlopment stages of the GIMME library in order for the developer to analize
    /// the status of objects while GIMME is working.
  
    class MERLIN_Object
    {
    public :
        /// Destructors of parent classes must always be virtual.
        virtual ~MERLIN_Object() {}
        /// This is the method child classes must implement in order to use the overloaded << operator.
        virtual void	dump( ostream &strm ) const = 0 ;
    };   

    inline ostream &
    operator<<( ostream &strm, const MERLIN_Object &obj )
    {
        obj.dump( strm );
        return strm;
    };

} // end namespace CSAC

#endif // MERLINObj_h

