/* ============================================================================
 * Language: C++ 
 * File: MERLIN_ParametersLimits.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#ifndef MERLIN_ParametersLimits_h_
#define MERLIN_ParametersLimits_h_ 1


#include <string>
using std::string; 

namespace CSAC
{
  

    class MERLIN_ParametersLimits
    {
	///  Minimum Line strength (dimensionless)
	double _min_line_strength ;              
	///  Minimum Doppler width (milli Angstroms)
	double _min_doppler_width ;              
	///  Minimum Damping (dimensionless)
	double _min_damping;          
	///  Minimum Planck function (same units as intensity)
	double _min_planck_function ;            
	///  Minimum Planck function gradient (same units as intensity)
	double _min_planck_function_gradient ;  
	/// Minimum Macroturbulence (km/s)
	double _min_vmac;
	/// Minimum Stray light (0 -> 1)
	double _min_stray_light ;   
	/// Minimum Shift on the stray light 
	double _min_stray_light_shift;            
	///  Maximum Magnetic field strength (Gauss)
	double _max_magnetic_field ;        
	///  Maximum Line strength (dimensionless)
	double _max_line_strength ;              
	///  Maximum Doppler width (milli Angstroms)
	double _max_doppler_width ;              
	///  Maximum Damping (dimensionless)
	double _max_damping;          
	///  Maximum Planck function (same units as intensity)
	double _max_planck_function ;            
	///  Maximum Planck function gradient (same units as intensity)
	double _max_planck_function_gradient ;  
	/// Maximum Macroturbulence (km/s)
	double _max_vmac;
	/// Maximum Stray light (0 -> 1)
	double _max_stray_light ;   
	/// Maximum Shift on the stray light 
	double _max_stray_light_shift;    
	///
	bool _max_planck_function_set;
	///
	bool _max_planck_function_gradient_set;
    public:
        /// Basic Cosntructor
        /// Set predetermined limits for an inversion
        MERLIN_ParametersLimits();
	///
	void set_max_magnetic_field(const double & val) throw (string);
	///
	void set_min_line_strength(const double & val) throw (string);
	///
	void set_max_line_strength(const double & val) throw (string);
	///
	void set_min__doppler_width(const double & val)throw (string);
	///
	void set_max_doppler_width(const double & val)throw (string);
	///
	void set_min_damping(const double & val) throw (string);
	///
	void set_max_damping(const double & val) throw (string);
	///
	void set_min_vmac(const double & val) throw (string);
	///
	void set_max_vmac(const double & val) throw (string);
	///
	void set_min_planck_function(const double & val) throw (string);
	///
	void set_max_planck_function(const double & val) throw (string);
	///
	void set_min_planck_function_gradient(const double & val) throw (string);
	///
	void set_max_planck_function_gradient(const double & val) throw (string);
	
	///
	bool set() const
	{ 
            if ( _max_planck_function_set && _max_planck_function_gradient_set )
                return true;
            return false;
	}
	
	
	/// Only this class can use this private data members;
	friend class MERLIN_ModelCorrection;
	
    };

} // end namespace CSAC

#endif // MERLIN_ParametersLimits_h_

