/* ============================================================================
 * Language: C++ 
 * File: MERLIN_PrintFormated.cc
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */

#include <stdio.h>

#include <string.h>
#include <stdlib.h>

#include <iostream>
using std::endl;
using std::cerr;
using std::flush;
using std::ostream;
#include <string>
using std::string;


#include "MERLIN_PrintFormated.h"
using CSAC::MERLIN_PrintFormated;

unsigned int MERLIN_PrintFormated::block_size=20;

void MERLIN_PrintFormated::print_blocked(ostream &oss, string s)
{
    int l=s.length();
    if (l>block_size)
    {
        char errs[256] ;
        sprintf( errs, "%s %d\n%s %s\n",
                 "Trying to print block larger than",
                 block_size,
                 "Current block is ", s.c_str() ) ;
        cerr<<errs<<endl;
        exit(1);
    }
    for (int q=0; q<(block_size-l); q++)
        oss<<" ";
    oss<<s<<flush;
}

void MERLIN_PrintFormated::print_blocked(ostream &oss, double num)
{
	char *s;
    s = new char [block_size];
    snprintf(s, sizeof(s), "%.12f", num);
    int l=strlen(s);
    if (l>block_size)
    {
        char errs[256] ;
        sprintf( errs, "%s %d\n%s %s\n",
                 "Trying to print block larger than",
                 block_size,
                 "Current block is ", s ) ;
        cerr<<errs<<endl;
        exit(1);
    }
    for (int q=0; q<(block_size-l); q++)
        oss<<" ";
    oss<<s<<flush;
	delete [] s;
}

void MERLIN_PrintFormated::print_blocked(ostream &oss, float num)
{
	char *s;
    s = new char [block_size];
    snprintf(s, sizeof(s), "%.12f", num);
    int l=strlen(s);
    if (l>block_size)
    {
        char errs[256] ;
        sprintf( errs, "%s %d\n%s %s\n",
                 "Trying to print block larger than",
                 block_size,
                 "Current block is ", s ) ;
        cerr<<errs<<endl;
        exit(1);
    }
    for (int q=0; q<(block_size-l); q++)
        oss<<" ";
    oss<<s<<flush;
	delete [] s;
}

void MERLIN_PrintFormated::print_blocked(ostream &oss, int num)
{
	char *s;
    s = new char [block_size];
    snprintf(s, sizeof(s), "%i", num);
    int l=strlen(s);
    if (l>block_size)
    {
        char errs[256] ;
        sprintf( errs, "%s %d\n%s %s\n",
                 "Trying to print block larger than",
                 block_size,
                 "Current block is ", s ) ;
        cerr<<errs<<endl;
        exit(1);
    }
    for (int q=0; q<(block_size-l); q++)
        oss<<" ";
    oss<<s<<flush;
	delete [] s;
}

