/* ============================================================================
 * Language: C++ 
 * File: MERLIN_Voigt.h
 * Package Name: Milne Eddington gRid Linear Inversion Network (MERLIN).
 * Sub-package: Fundamental Inversion Code (FICO).
 *
 * Created by Jose Garcia <jgarcia@ucar.edu> on 11/11/05.
 *  
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact University Corporation for Atmospheric Research at
 * 3080 Center Green Drive, Boulder, CO 80301

 * (c) COPYRIGHT University Corporation for Atmostpheric Research 2004-2006
 * Please read the full copyright statement in the file COPYRIGHT_UCAR.
 *
 =========================================================================== */


#ifndef MERLIN_Voigt_h_
#define MERLIN_Voigt_h_ 1

#define  my_voigt(damping,frequency_offset,voigt_value,faraday_value)   \
    {                                                                   \
        if (frequency_offset < 0)                                       \
            CSAC::VOIGT::ivsigno=-1;                                    \
        else                                                            \
            CSAC::VOIGT::ivsigno=1;                                     \
        CSAC::VOIGT::V = CSAC::VOIGT::ivsigno * frequency_offset;       \
        if (damping == 0)                                               \
        {                                                               \
            voigt_value = exp(-(CSAC::VOIGT::V * CSAC::VOIGT::V));      \
            if (CSAC::VOIGT::V > CSAC::VOIGT::XDWS[0])                  \
            {                                                           \
                if (CSAC::VOIGT::V > CSAC::VOIGT::XDWS[27])             \
                {                                                       \
                    CSAC::VOIGT::Y = 0.5 / CSAC::VOIGT::V;              \
                    CSAC::VOIGT::D = CSAC::VOIGT::Y * (1 + CSAC::VOIGT::Y / CSAC::VOIGT::V); \
                }                                                       \
                else                                                    \
                {                                                       \
                    CSAC::VOIGT::k = 26;                                \
                    CSAC::VOIGT::i = 1;                                 \
                    while(CSAC::VOIGT::i < 27)                          \
                    {                                                   \
                        if (CSAC::VOIGT::XDWS[CSAC::VOIGT::i] < CSAC::VOIGT::V) \
                            CSAC::VOIGT::i++;                           \
                        else                                            \
                        {                                               \
                            CSAC::VOIGT::k = CSAC::VOIGT::i;            \
                            break;                                      \
                        }                                               \
                    }                                                   \
                    CSAC::VOIGT::kk = CSAC::VOIGT::k-1;                 \
                    CSAC::VOIGT::kkk = CSAC::VOIGT::k+1;                \
                    CSAC::VOIGT::D1 = CSAC::VOIGT::V - CSAC::VOIGT::XDWS[CSAC::VOIGT::kk]; \
                    CSAC::VOIGT::D2 = CSAC::VOIGT::V - CSAC::VOIGT::XDWS[CSAC::VOIGT::k]; \
                    CSAC::VOIGT::D3 = CSAC::VOIGT::V - CSAC::VOIGT::XDWS[CSAC::VOIGT::kkk]; \
                    CSAC::VOIGT::D12 = CSAC::VOIGT::XDWS[CSAC::VOIGT::kk] -CSAC::VOIGT::XDWS[CSAC::VOIGT::k]; \
                    CSAC::VOIGT::D13 = CSAC::VOIGT::XDWS[CSAC::VOIGT::kk] - CSAC::VOIGT::XDWS[CSAC::VOIGT::kkk]; \
                    CSAC::VOIGT::D23 = CSAC::VOIGT::XDWS[CSAC::VOIGT::k] - CSAC::VOIGT::XDWS[CSAC::VOIGT::kkk]; \
                    CSAC::VOIGT::D =                                    \
                    CSAC::VOIGT::YDWS[CSAC::VOIGT::kk] * CSAC::VOIGT::D2 * CSAC::VOIGT::D3 / (CSAC::VOIGT::D12 * CSAC::VOIGT::D13) - \
                    CSAC::VOIGT::YDWS[CSAC::VOIGT::k] * CSAC::VOIGT::D1 * CSAC::VOIGT::D3 / (CSAC::VOIGT::D12 * CSAC::VOIGT::D23) + \
                    CSAC::VOIGT::YDWS[CSAC::VOIGT::k] * CSAC::VOIGT::D1 * CSAC::VOIGT::D2 / (CSAC::VOIGT::D13 * CSAC::VOIGT::D23); \
                }                                                       \
            }                                                           \
            else                                                        \
                CSAC::VOIGT::D = CSAC::VOIGT::V * (1. -.66666667 * (CSAC::VOIGT::V * CSAC::VOIGT::V)); \
            faraday_value = CSAC::VOIGT::D * CSAC::VOIGT::ivsigno * 5.641895836E-1; \
        }                                                               \
        else                                                            \
        {                                                               \
            CSAC::VOIGT::Z1_real = CSAC::VOIGT::A6 * damping + CSAC::VOIGT::A5; \
            CSAC::VOIGT::Z1_imag = CSAC::VOIGT::A6 * -CSAC::VOIGT::V;   \
            CSAC::VOIGT::Z2_real = CSAC::VOIGT::Z1_real * damping - CSAC::VOIGT::Z1_imag * -CSAC::VOIGT::V        + CSAC::VOIGT::A4; \
            CSAC::VOIGT::Z2_imag = CSAC::VOIGT::Z1_real * -CSAC::VOIGT::V      + CSAC::VOIGT::Z1_imag * damping; \
            CSAC::VOIGT::Z3_real = CSAC::VOIGT::Z2_real * damping - CSAC::VOIGT::Z2_imag * -CSAC::VOIGT::V        + CSAC::VOIGT::A3; \
            CSAC::VOIGT::Z3_imag = CSAC::VOIGT::Z2_real * -CSAC::VOIGT::V      + CSAC::VOIGT::Z2_imag * damping; \
            CSAC::VOIGT::Z4_real = CSAC::VOIGT::Z3_real * damping - CSAC::VOIGT::Z3_imag * -CSAC::VOIGT::V        + CSAC::VOIGT::A2; \
            CSAC::VOIGT::Z4_imag = CSAC::VOIGT::Z3_real * -CSAC::VOIGT::V      + CSAC::VOIGT::Z3_imag * damping; \
            CSAC::VOIGT::Z5_real = CSAC::VOIGT::Z4_real * damping - CSAC::VOIGT::Z4_imag * -CSAC::VOIGT::V        + CSAC::VOIGT::A1; \
            CSAC::VOIGT::Z5_imag = CSAC::VOIGT::Z4_real * -CSAC::VOIGT::V      + CSAC::VOIGT::Z4_imag * damping; \
            CSAC::VOIGT::Z6_real = CSAC::VOIGT::Z5_real * damping - CSAC::VOIGT::Z5_imag * -CSAC::VOIGT::V        + CSAC::VOIGT::A0; \
            CSAC::VOIGT::Z6_imag = CSAC::VOIGT::Z5_real * -CSAC::VOIGT::V      + CSAC::VOIGT::Z5_imag * damping; \
            CSAC::VOIGT::ZZ1_real = damping + CSAC::VOIGT::B6;          \
            CSAC::VOIGT::ZZ1_imag = -CSAC::VOIGT::V;                    \
            CSAC::VOIGT::ZZ2_real = CSAC::VOIGT::ZZ1_real * damping - CSAC::VOIGT::ZZ1_imag * -CSAC::VOIGT::V     + CSAC::VOIGT::B5; \
            CSAC::VOIGT::ZZ2_imag = CSAC::VOIGT::ZZ1_real * -CSAC::VOIGT::V      + CSAC::VOIGT::ZZ1_imag * damping; \
            CSAC::VOIGT::ZZ3_real = CSAC::VOIGT::ZZ2_real * damping - CSAC::VOIGT::ZZ2_imag * -CSAC::VOIGT::V     + CSAC::VOIGT::B4; \
            CSAC::VOIGT::ZZ3_imag = CSAC::VOIGT::ZZ2_real * -CSAC::VOIGT::V      + CSAC::VOIGT::ZZ2_imag * damping; \
            CSAC::VOIGT::ZZ4_real = CSAC::VOIGT::ZZ3_real * damping - CSAC::VOIGT::ZZ3_imag * -CSAC::VOIGT::V     + CSAC::VOIGT::B3; \
            CSAC::VOIGT::ZZ4_imag = CSAC::VOIGT::ZZ3_real * -CSAC::VOIGT::V      + CSAC::VOIGT::ZZ3_imag * damping; \
            CSAC::VOIGT::ZZ5_real = CSAC::VOIGT::ZZ4_real * damping - CSAC::VOIGT::ZZ4_imag * -CSAC::VOIGT::V     + CSAC::VOIGT::B2; \
            CSAC::VOIGT::ZZ5_imag = CSAC::VOIGT::ZZ4_real * -CSAC::VOIGT::V      + CSAC::VOIGT::ZZ4_imag * damping; \
            CSAC::VOIGT::ZZ6_real = CSAC::VOIGT::ZZ5_real * damping  - CSAC::VOIGT::ZZ5_imag * -CSAC::VOIGT::V    + CSAC::VOIGT::B1; \
            CSAC::VOIGT::ZZ6_imag = CSAC::VOIGT::ZZ5_real * -CSAC::VOIGT::V       + CSAC::VOIGT::ZZ5_imag * damping; \
            CSAC::VOIGT::ZZ7_real = CSAC::VOIGT::ZZ6_real * damping  - CSAC::VOIGT::ZZ6_imag * -CSAC::VOIGT::V    + CSAC::VOIGT::B0; \
            CSAC::VOIGT::ZZ7_imag = CSAC::VOIGT::ZZ6_real * -CSAC::VOIGT::V       + CSAC::VOIGT::ZZ6_imag * damping; \
            CSAC::VOIGT::division_factor = 1 / (CSAC::VOIGT::ZZ7_real * CSAC::VOIGT::ZZ7_real + CSAC::VOIGT::ZZ7_imag * CSAC::VOIGT::ZZ7_imag); \
            CSAC::VOIGT::ZZZ_real =                                     \
            (CSAC::VOIGT::Z6_real * CSAC::VOIGT::ZZ7_real  + CSAC::VOIGT::Z6_imag * CSAC::VOIGT::ZZ7_imag) * \
            CSAC::VOIGT::division_factor;                               \
            CSAC::VOIGT::ZZZ_imag =                                     \
            (CSAC::VOIGT::Z6_real * -CSAC::VOIGT::ZZ7_imag + CSAC::VOIGT::Z6_imag * CSAC::VOIGT::ZZ7_real) * \
            CSAC::VOIGT::division_factor;                               \
            voigt_value=CSAC::VOIGT::ZZZ_real;                          \
            faraday_value=.5 * CSAC::VOIGT::ivsigno * CSAC::VOIGT::ZZZ_imag; \
        }                                                               \
    }

namespace CSAC
{
    namespace VOIGT
    {
        /* coefficients of the rational approximation formula
           to the complementary error function 
        */ 
        extern const double A0;
        extern const double A1;
        extern const double A2;
        extern const double A3;
        extern const double A4;
        extern const double A5;
        extern const double A6;
        extern const double B0;
        extern const double B1;
        extern const double B2;
        extern const double B3;
        extern const double B4;
        extern const double B5;
        extern const double B6;
        extern const double XDWS[];
        extern const double YDWS[];
        extern int i;
        extern int k;
        extern int kk; 
        extern int kkk;
        extern int ivsigno;
        extern double V;
        extern double Y;
        extern double D;
        extern double D1;
        extern double D2;
        extern double D3;
        extern double D12; 
        extern double D13; 
        extern double D23;
        extern double Z1_real ;
        extern double Z1_imag;
        extern double Z2_real;
        extern double Z2_imag;
        extern double Z3_real;
        extern double Z3_imag;
        extern double Z4_real;
        extern double Z4_imag;
        extern double Z5_real;
        extern double Z5_imag;
        extern double Z6_real;
        extern double Z6_imag;
        extern double ZZ1_real;
        extern double ZZ1_imag;
        extern double ZZ2_real;
        extern double ZZ2_imag;
        extern double ZZ3_real;
        extern double ZZ3_imag;
        extern double ZZ4_real;
        extern double ZZ4_imag;
        extern double ZZ5_real;
        extern double ZZ5_imag;
        extern double ZZ6_real;
        extern double ZZ6_imag;
        extern double ZZ7_real;
        extern double ZZ7_imag;
        extern double division_factor;
        extern double ZZZ_real;
        extern double ZZZ_imag;
    
    }; // end namespace CSAC::voigt
} // end namespace CSAC
	      
namespace CSAC
{
    namespace VOIGT
    {
        enum v_type{FORTRAN=0, CPLUSPLUS};
    
        void call_voigt(
                        double *damping, 
                        double *frequency_offset, 
                        double *voigt, 
                        double *faraday,
                        enum v_type type
                        );
    }
} // end namespace CSAC

#endif // MERLIN_Voigt_h_
