!!%%%%%%%%%%%%%%%%%%%%%%%%
!! SUBROUTINE ZEEMAN_COMP
!!%%%%%%%%%%%%%%%%%%%%%%%%
!! J.M.BORRERO (29.07.04)
!!%%%%%%%%%%%%%%%%%%%%%%%%

SUBROUTINE ZEEMAN_COMP(BFIELD,LANDA0,JL,JU,SL,SU,LL,LU,NTOT,ENG,STR,LOC,DEDB)
!------------------
USE VALUES
!------------------
IMPLICIT NONE
INTEGER(I4B)                  :: SL,LL,JL,SU,LU,JU
INTEGER(I4B)                  :: NMJL,NMJU,NPI,NSR,NSB,NTOT
INTEGER(I4B)                  :: I,J,K,DELTAM,DELTAJ,HSM(1)
REAL(DP)                      :: BFIELD,LANDA0,GU,GL,GEFF
REAL(DP)                      :: DELTAL,DELTALDB,S,SPI,SSR,SSB
REAL(DP)         ,ALLOCATABLE :: ML(:),MU(:)
REAL(DP)         ,ALLOCATABLE :: NEWENG(:),NEWSTR(:),NEWLOC(:),NEWDEDB(:)
REAL(DP)                      :: ENG(NTOT),STR(NTOT),LOC(NTOT),DEDB(NTOT)
!------------------
I=1
!------------------
! Lande Factors
IF (JU.NE.0) THEN
   GU=1.5D0+(SU*(SU+1D0)-LU*(LU+1D0))/(2D0*JU*(JU+1D0))
ELSE
   GU=0D0
ENDIF
IF (JL.NE.0) THEN
   GL=1.5D0+(SL*(SL+1D0)-LL*(LL+1D0))/(2D0*JL*(JL+1D0))
ELSE
   GL=0D0
ENDIF
GEFF=0.5D0*(GL+GU)+0.25D0*(GL-GU)*(JL*(JL+1D0)-JU*(JU+1D0))
! New energy levels
NMJL=2*JL+1        ! Number of lower levels 
NMJU=2*JU+1        ! Number of upper levels 

! Allocating needed arrays
ALLOCATE(ML(NMJL),MU(NMJU))
ALLOCATE(NEWENG(NTOT),NEWSTR(NTOT),NEWLOC(NTOT),NEWDEDB(NTOT))
! M QUANTUM NUMBERS
DO J=1,NMJL
   ML(J)=(J-1D0)-JL
ENDDO
DO J=1,NMJU
   MU(J)=(J-1D0)-JU
ENDDO
! CHECKING ALLOWED TRANSITIONS
DO K=1,NMJL
   DO J=1,NMJU
      DELTAM=MU(J)-ML(K)
      IF (ABS(DELTAM).LE.1) THEN 
         IF (I.LE.NTOT) THEN
         DELTAL=4.66685E-10*(ML(K)*GL-MU(J)*GU)*BFIELD*LANDA0**2D0 ! (mA)
         DELTALDB=4.66685E-10*(ML(K)*GL-MU(J)*GU)*LANDA0**2D0      ! (mA/GAUSS)
         ENG(I)=DELTAL+LANDA0
         DEDB(I)=DELTALDB
         DELTAJ=JU-JL
         LOC(I)=DELTAM
         !IF (ABS(DELTAJ).GT.1) THEN CALL ERROR(1)
         SELECT CASE (DELTAJ)
                CASE(-1)
                   SELECT CASE (DELTAM)
                      CASE(-1)
                         S=(JL+MU(J))*(JU+MU(J)+2D0)
                      CASE(0)
                         S=2D0*(JL**2D0-MU(J)**2D0)
                      CASE(1)
                         S=(JL-MU(J))*(JU-MU(J)+2D0)
                      END SELECT
                CASE(0)
                   SELECT CASE (DELTAM)
                      CASE(-1)
                         S=(JU-MU(J))*(JU+MU(J)+1D0)
                      CASE(0)
                         S=2D0*MU(J)**2D0
                      CASE(1)
                         S=(JU+MU(J))*(JU-MU(J)+1D0)
                   END SELECT
                CASE(1)
                   SELECT CASE (DELTAM)
                      CASE(-1)
                         S=(JU-MU(J))*(JL-MU(J))
                      CASE(0)
                         S=2D0*(JU**2D0-MU(J)**2D0)
                      CASE(1)
                         S=(JU+MU(J))*(JL+MU(J))
                   END SELECT
         END SELECT
         STR(I)=S
         I=I+1
         ENDIF
    ENDIF
   ENDDO
ENDDO

! NORMALIZATION OF STRENGHTS IN THE ZEEMAN PATTERN
SPI=0D0
SSR=0D0
SSB=0D0
   DO J=1,NTOT
      IF (LOC(J).LT.0D0) SSR=SSR+STR(J)
      IF (LOC(J).GT.0D0) SSB=SSB+STR(J)
      IF (LOC(J).LT.0.5D0) THEN
         IF (LOC(J).GT.-0.5D0) SPI=SPI+STR(J)
      ENDIF
   ENDDO
   DO J=1,NTOT
      IF (LOC(J).LT.0D0) STR(J)=STR(J)/SSR
      IF (LOC(J).GT.0D0) STR(J)=STR(J)/SSB
      IF (LOC(J).LT.0.5D0) THEN
         IF (LOC(J).GT.-0.5D0) STR(J)=STR(J)/SPI
      ENDIF
   ENDDO
! ORDERING IN WAVELENGTHS
DO J=1,NTOT
HSM=MINLOC(ENG)
NEWENG(J)=ENG(HSM(1))
NEWSTR(J)=STR(HSM(1))
NEWLOC(J)=LOC(HSM(1))
NEWDEDB(J)=DEDB(HSM(1))
ENG(HSM)=ENG(HSM(1))*1000D0
ENDDO
ENG=NEWENG
STR=NEWSTR
LOC=NEWLOC
DEDB=NEWDEDB

! DEALLOCATING NOT NEEDED ARRAYS
DEALLOCATE(NEWENG,NEWLOC,NEWSTR,NEWDEDB)
DEALLOCATE(ML,MU)
! We do not deallocate ENG,LOC,STR and DEDB becuase we need to give them
! to absortion.f90. 
END SUBROUTINE ZEEMAN_COMP

