	subroutine ruffazm
     .	( np, obsq, obsu
     .	, limblu, ceni, limred, dvdmin
     .	, dirazm
     .	)

!	subroutine:  ruffazam
!
!	purpose:  Find prefered azimuth angle by least-squares method of
!		  Auer Heasley House (1977,Solar Physics, 55, 47).
!
!	author:  paul@ncar  (revised 3/97)
!
!	input:
!		  np	- dimension of input data arrays.
!	      obsq(np)	- observed q-profile.
!	      obsu(np)	- observed u-profile.
!
!		limblu  - lower limit of band pass, pixels (integer).
!		ceni	- approximate spectra line center, pixels (real).
!		limred  - upper limit of band pass, pixels (integer).
!		dvdmin  - minimum doppler width, pixels (real).
!	output:
!		dirazm	- prefered azimuth, radians.
!
!-------------------------------------------------------------------------------
	integer w

	parameter(  pi = 3.14159 26535 89793 23846 )

				    !Input data arrays.
	dimension obsq(np), obsu(np)

				    !Temporary arrays.
	parameter( itmp = 2048 )
	dimension vec1(itmp), vec2(itmp)

	if( itmp .lt. np ) then
		write(6,'(''a.ruffazm.f: itmp dimension to small'')')
		stop 'a.ruffazm.f: itmp dimension to small'
	endif
				    !Compute arrays used in least squares.
	do 600 w = limblu,limred
		vec1(w) = obsq(w)*obsu(w)
		vec2(w) = obsq(w)**2-obsu(w)**2
600	continue

	snum = 0.
	do 100 w = limblu,limred
		snum = snum+vec1(w)
100	continue

	sden = 0.
	do 110 w = limblu,limred
		sden = sden+vec2(w)
110	continue
				    !Angle that gives minimum or maximum
				    !sum of squared u.
	dirazm = pi/8.
	if(sden.ne.0.) dirazm = .25*atan(2.*snum/sden)

				    !Resolve pi/4 ambiguity.
				    !If second derivative of sum u*u is
				    !negative (for maximum) add pi/4.
	if(.5*sden*cos(4.*dirazm)+snum*sin(4.*dirazm).lt.0.)
     .	dirazm = dirazm+pi/4.
				    !Resolve pi/2 ambiquity.
				    !Look for dip in middle of q-profile.
				    !If second derivative of q-profile is
				    !negative in line core add pi/2.

				    !Rotated q-profile to azimuth.
	sin2azm = sin(2.*dirazm)
	cos2azm = cos(2.*dirazm)
	do 200 w = 1,np
		vec2(w) = obsq(w)*cos2azm+obsu(w)*sin2azm
200	continue
				    !First derivative of q-profile.
	do 211 w = 2,np-1
		vec1(w) = vec2(w+1)-vec2(w-1)
211	continue
				    !Second derivative of q-profile.
	do 212 w = 3,np-2
		vec2(w) = vec1(w+1)-vec1(w-1)
212	continue
				    !Sum of second derivative about center
				    !to get trend.
	sum = 0.
	jcen = nint(ceni)
	i2dvd = 2.*dvdmin
	do 213 w = max(limblu,jcen-i2dvd,3),min(limred,jcen+i2dvd,np-2)
		sum = sum+vec2(w)
213	continue

	if(sum.lt.0.) dirazm = dirazm+pi/2.

				    !Put azimuth in range -pi/2 to pi/2.
	if(dirazm.gt. pi) dirazm =  amod( dirazm,pi)
	if(dirazm.lt.-pi) dirazm = -amod(-dirazm,pi)
	if(dirazm.gt. pi/2.) dirazm = dirazm-pi
	if(dirazm.lt.-pi/2.) dirazm = dirazm+pi

	return
	end
