	subroutine ruffcsn
     .	( np, obsq, obsu, obsv
     .	, limblu, ceni, limred
     .	, direta0, dircsn
     .	)
!
!	subroutine: ruffcsn
!
!	purpose:  Find ruff estimate of the cosine of the field inclination
!		  away from the line of sight; also get estimate of eta0.
!
!	author:  paul@ncar  (revised 2/97)
!
!	input:
!		  np	- dimension of input data arrays.
!	      obsq(np)	- observed q-profile.
!	      obsu(np)	- observed u-profile.
!	      obsv(np)	- observed v-profile.
!
!		limblu  - lower limit of band pass, pixels (integer)
!			  (may be internally extended).
!		ceni	- approximate spectra line center, pixels (real).
!		limred  - upper limit of band pass, pixels (integer)
!			  (may be internally extended).
!
!	output:
!		dircsn	- ruff cosine of field angle from line of sight.
!		direta0	- ruff eta0.
!
!------------------------------------------------------------------------------
	integer w

				    !Data arrays.
	dimension obsq(np), obsu(np), obsv(np)

				    !Temporary storage.
	parameter( itmp = 2048 )
	dimension qquu(itmp)
	if( itmp .lt. np ) then
		write(6,'(''a.ruffcsn.f: itmp dimension to small'')')
		stop 'a.ruffcsn.f: itmp dimension to small'
	endif
				    !Broaden band pass because telluric
				    !lines are not present.
	jcen = nint( ceni )
	jj   = 2*max( iabs(limblu-jcen), iabs(limred-jcen) )
	jblu = max( 1, jcen-jj )
	jred = min( jcen+jj, np )
				    !Find maximum amplitude of v-profile.
	call findmx(obsv,jcen,jred,vmax,vmin,jmx,jmn)
	if(abs(vmax).gt.abs(vmin)) then
		vobs = vmax
		maxv = jmx
	else
		vobs = vmin
		maxv = jmn
	endif
	vampl = abs(vobs)
				    !Amplitude of q,u signal.
	do 100 w = jblu,jred
		qquu(w) = sqrt(obsq(w)**2+obsu(w)**2)
100	continue
				    !Look for transition of q,u amplitude
				    !in wing of line.
	call findmx(qquu,jcen,maxv,gunk1,gunk2,junk1,jwing)

				    !Local maximum q,u amplitude in wing
				    !of line.
	kred = min(maxv+maxv-jwing,jred)
	call findmx(qquu,jwing,kred,qumax,gunk2,junk1,junk2)

				    !Approximate cosine inclination angle.
	if( vampl .ne. 0. ) then
		r = 2.*qumax/vampl
		dircsn = .5*(sqrt(r*r+4.)-r)
	else
		dircsn = 0.
	endif
				    !Resolve field reversal.
	sumb = 0.
	do 200 w = jblu,jcen
		sumb = sumb+obsv(w)
200	continue
	sumr = 0.
	do 201 w = jcen,jred
		sumr = sumr+obsv(w)
201	continue
	if(sumr.gt.sumb) dircsn = -dircsn

				    !Bound cosine of angle.
	dircsn = amax1(-.99996,amin1(.99996,dircsn))

				    !Very crude estimate of eta0.
	direta0 = 15.
	if(qquu(jcen).lt.qumax) direta0 = 10.

	return
	end
